/*
 * Copyright (c) 2002-2007, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.jcr.business;

import java.io.File;

import java.security.PrivilegedAction;


/**
 * Utility class that wrapp some RepositoryFileHome method
 * in PrivilegedAction
 */
public final class RestrictedRepositoryFileHome
{
    /**
     * Empty private constructor
     */
    private RestrictedRepositoryFileHome(  )
    {
    }

    /**
     * @param instance a RepositoryFileHome instance
     * @param strWorkspace the workspace name
     * @param path the absolute file path
     * @param file the file content
     * @return an action to be runned in a Restricted context
     */
    public static PrivilegedAction addFile( final RepositoryFileHome instance, final String strWorkspace,
        final String path, final File file )
    {
        return new PrivilegedAction(  )
            {
                public Object run(  )
                {
                    instance.addFile( strWorkspace, path, file );

                    return null;
                }
            };
    }

    /**
     * @param instance a RepositoryFileHome instance
     * @param strWorkspace the workspace name
     * @param path the absolute directory path
     * @return an action to be runned in a Restricted context
     */
    public static PrivilegedAction createFolder( final RepositoryFileHome instance, final String strWorkspace,
        final String path )
    {
        return new PrivilegedAction(  )
            {
                public Object run(  )
                {
                    instance.createFolder( strWorkspace, path );

                    return null;
                }
            };
    }

    /**
     * @param instance a RepositoryFileHome instance
     * @param strWorkspace the workspace name
     * @param strFileId the absolute path file
     * @return an action to be runned in a Restricted context
     */
    public static PrivilegedAction getRepositoryFileById( final RepositoryFileHome instance, final String strWorkspace,
        final String strFileId )
    {
        return new PrivilegedAction(  )
            {
                public Object run(  )
                {
                    return instance.getRepositoryFileById( strWorkspace, strFileId );
                }
            };
    }

    /**
     * @param instance a RepositoryFileHome instance
     * @param strWorkspace the workspace name
     * @param path the absolute path file
     * @return an action to be runned in a Restricted context
     */
    public static PrivilegedAction getRepositoryFile( final RepositoryFileHome instance, final String strWorkspace,
        final String path )
    {
        return new PrivilegedAction(  )
            {
                public Object run(  )
                {
                    return instance.getRepositoryFile( strWorkspace, path );
                }
            };
    }

    /**
     * @param instance a RepositoryFileHome instance
     * @param strWorkspace the workspace name
     * @param parent the parent directory path
     * @return an action to be runned in a Restricted context
     */
    public static PrivilegedAction getRepositoryFileList( final RepositoryFileHome instance, final String strWorkspace,
        final String parent )
    {
        return new PrivilegedAction(  )
            {
                public Object run(  )
                {
                    return instance.getRepositoryFileList( strWorkspace, parent );
                }
            };
    }

    /**
     * @param instance a RepositoryFileHome instance
     * @param strWorkspace the workspace name
     * @return an action to be runned in a Restricted context
     */
    public static PrivilegedAction getRepositoryFileList( final RepositoryFileHome instance, final String strWorkspace )
    {
        return new PrivilegedAction(  )
            {
                public Object run(  )
                {
                    return instance.getRepositoryFileList( strWorkspace );
                }
            };
    }

    /**
     * @param instance a RepositoryFileHome instance
     * @param strWorkspace the workspace name
     * @param strPath an absolute path
     * @return an action to be runned in a Restricted context
     */
    public static PrivilegedAction removeRepositoryFile( final RepositoryFileHome instance, final String strWorkspace,
        final String strPath )
    {
        return new PrivilegedAction(  )
            {
                public Object run(  )
                {
                    instance.removeRepositoryFile( strWorkspace, strPath );

                    return null;
                }
            };
    }
}
