/*
 * Copyright (c) 2002-2007, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.jcr.business.admin;

import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.spring.SpringContextService;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;


/**
 * Home class for JCR administration
 */
public final class AdminJcrHome
{
    private static IAdminViewDAO _adminViewDao = (IAdminViewDAO) SpringContextService.getPluginBean( "jsr170",
            "adminViewDAO" );
    private static IAdminWorkspaceDAO _adminWorkspaceDao = (IAdminWorkspaceDAO) SpringContextService.getPluginBean( "jsr170",
            "adminWorkspaceDAO" );
    private static AdminJcrHome _instance;

    /**
     * Private constructor
     */
    private AdminJcrHome(  )
    {
    }

    /**
     * @return an instance of AdminJcrHome
     */
    public static AdminJcrHome getInstance(  )
    {
        if ( _instance == null )
        {
            _instance = new AdminJcrHome(  );
        }

        return _instance;
    }

    /**
     * @param workspace the workspace to create
     * @param plugin the plugin
     */
    public void createWorkspace( AdminWorkspace workspace, Plugin plugin )
    {
        _adminWorkspaceDao.insert( workspace, plugin );
    }

    /**
     * @param workspace the workspace to modify
     * @param plugin the plugin
     */
    public void modifyWorkspace( AdminWorkspace workspace, Plugin plugin )
    {
        _adminWorkspaceDao.store( workspace, plugin );
    }

    /**
     * @param id of the workspace to delete
     * @param plugin the plugin
     */
    public void deleteWorkspace( int id, Plugin plugin )
    {
        _adminWorkspaceDao.delete( id, plugin );
    }

    /**
     * @param view the view to create
     * @param plugin the plugin
     */
    public void createView( AdminView view, Plugin plugin )
    {
        _adminViewDao.insert( view, plugin );
    }

    /**
     * @param view the view to modify
     * @param plugin the plugin
     */
    public void modifyView( AdminView view, Plugin plugin )
    {
        _adminViewDao.store( view, plugin );
    }

    /**
     * @param id the id of the view to delete
     * @param plugin the plugin
     */
    public void deleteView( int id, Plugin plugin )
    {
        _adminViewDao.delete( id, plugin );
    }

    /**
     * @param plugin the plugin
     * @return all available views
     */
    public Collection<AdminView> findAllViews( Plugin plugin )
    {
        return _adminViewDao.selectAll( plugin );
    }

    /**
     * @param plugin the plugin
     * @return all available workspaces
     */
    public Collection<AdminWorkspace> findAllWorkspaces( Plugin plugin )
    {
        return _adminWorkspaceDao.selectAll( plugin );
    }

    /**
     * @param plugin the plugin
     * @return amapp of all workspaces
     */
    public Map<String, AdminWorkspace> getWorkspacesList( Plugin plugin )
    {
        HashMap<String, AdminWorkspace> result = new HashMap<String, AdminWorkspace>(  );

        for ( AdminWorkspace workspace : _adminWorkspaceDao.selectAll( plugin ) )
        {
            result.put( String.valueOf( workspace.getId(  ) ), workspace );
        }

        return result;
    }

    /**
     * @param nWorkspaceId the workspace id to find
     * @param plugin the plugin
     * @return an AdminWorkspace
     */
    public AdminWorkspace findWorkspaceById( int nWorkspaceId, Plugin plugin )
    {
        return _adminWorkspaceDao.load( nWorkspaceId, plugin );
    }

    /**
     * @param nViewId the view ID to fin
     * @param plugin the plugin
     * @return the adminView
     */
    public AdminView findViewById( int nViewId, Plugin plugin )
    {
        return _adminViewDao.load( nViewId, plugin );
    }

    /**
     * @param nViewId the view ID to search the jcr type for
     * @param plugin the plugin
     * @return the jcr type
     */
    public String findJcrType( int nViewId, Plugin plugin )
    {
        AdminView view = findViewById( nViewId, plugin );
        AdminWorkspace workspace = findWorkspaceById( view.getWorkspaceId(  ), plugin );

        return workspace.getJcrType(  );
    }

    public boolean existsViewWithWorkspaceId( int nWorkspaceId, Plugin plugin )
    {
        return !_adminViewDao.selectAll( nWorkspaceId, plugin ).isEmpty(  );
    }
}
