/*
 * Copyright (c) 2002-2007, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.jcr.business.portlet;

import fr.paris.lutece.plugins.jcr.authentication.JcrRestrictedOperation;
import fr.paris.lutece.plugins.jcr.business.IRepositoryFile;
import fr.paris.lutece.plugins.jcr.business.IWorkspace;
import fr.paris.lutece.plugins.jcr.business.RepositoryFileHome;
import fr.paris.lutece.plugins.jcr.business.RestrictedRepositoryFileHome;
import fr.paris.lutece.plugins.jcr.business.admin.AdminJcrHome;
import fr.paris.lutece.plugins.jcr.business.admin.AdminView;
import fr.paris.lutece.plugins.jcr.business.admin.AdminWorkspace;
import fr.paris.lutece.portal.business.portlet.Portlet;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.plugin.PluginService;
import fr.paris.lutece.portal.service.security.LuteceUser;
import fr.paris.lutece.portal.service.security.SecurityService;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.web.constants.Parameters;
import fr.paris.lutece.util.xml.XmlUtil;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


/**
 * This class represents business objects Jsr170Portlet
 */
public class Jsr170Portlet extends Portlet
{
    ///////////////////////////////////////////////////////////////////////
    // Constants

    // Xml tags
    private static final String TAG_PORTLET_JSR170 = "jsr170-portlet";
    private static final String TAG_DIRECTORY_PATH = "directory-path";
    private static final String PARAMETER_WORKSPACE = "workspace";
    private static final String TAG_CURRENT_WORKSPACE = "current-workspace";
    private static final String TAG_WORKSPACE = "workspace";
    private static final String TAG_CAN_READ = "canRead";
    private static final String TAG_CAN_WRITE = "canWrite";
    private static final String TAG_CAN_REMOVE = "canRemove";
    private static final String TAG_WORKSPACES = "workspaces";
    private static final String TAG_BREADCRUMBS = "breadcrumbs";
    private static final String TAG_BREADCRUMB = "breadcrumb";
    private static final String TAG_BREADCRUMB_NAME = "breadcrumb-name";
    private static final String TAG_BREADCRUMB_PATH = "breadcrumb-path-id";
    private static final String TYPE_ALFRESCO = "ALFRESCO";
    private static final String PARAMETER_FILE = "file";
    private static final String PARAMETER_VIEW = "view";
    private static final String TAG_PORTLET_JSR170_ERROR = "jsr170-portlet-error";
    private static final String TAG_ERROR = "error";
    private static final String PARAMETER_FILE_ID = "file_id";

    // Variables declarations
    private int _nFolderListingFileId;
    private int _nCurrentView = -1;
    private boolean _hasDefaultView;
    private int _nDefaultView = -1;
    private String _strFileId = "";
    private String _strLogin;
    private String _strPassword;
    private Plugin _plugin;
    private AdminView _view;
    private JcrRestrictedOperation _jcrRestrictedOperation;

    /**
     * Creates a new Jsr170Portlet object.
     */
    public Jsr170Portlet(  )
    {
        setPortletTypeId( Jsr170PortletHome.getInstance(  ).getPortletTypeId(  ) );
    }

    /**
     * Sets the Id of the folderlistingfile
     *
     * @param nFolderListingFileId the new Id
     */
    public void setFolderListingFileId( int nFolderListingFileId )
    {
        _nFolderListingFileId = nFolderListingFileId;
    }

    /**
     * Returns the Id of this folderlistingfile
     *
     * @return the folderlistingfile Id
     */
    public int getFolderListingFileId(  )
    {
        return _nFolderListingFileId;
    }

    /**
     * Returns the Xml code of the FolderListing portlet without XML heading
     *
     * @param request The http request
     * @return the Xml code of the FolderListing portlet content
     */
    public String getXml( HttpServletRequest request )
    {
        StringBuffer sbXml = new StringBuffer(  );

        try
        {
            // retrieve the folder path from the request
            getParameters( request );

            final AdminJcrHome adminJcrInstance = AdminJcrHome.getInstance(  );
            AdminWorkspace adminWorkspace = adminJcrInstance.findWorkspaceById( _view.getWorkspaceId(  ), _plugin );
            final RepositoryFileHome instance = RepositoryFileHome.getInstance( adminWorkspace.getJcrType(  ) );
            final String currentWorkspace = adminWorkspace.getName(  );
            IRepositoryFile currentDir = null;

            if ( ( _strFileId == null ) || "".equals( _strFileId ) )
            {
                currentDir = (IRepositoryFile) _jcrRestrictedOperation.doRestrictedOperation( RestrictedRepositoryFileHome.getRepositoryFile( 
                            instance, currentWorkspace, "/" ) );
            }
            else
            {
                currentDir = (IRepositoryFile) _jcrRestrictedOperation.doRestrictedOperation( RestrictedRepositoryFileHome.getRepositoryFileById( 
                            instance, currentWorkspace, _strFileId ) );
            }

            XmlUtil.beginElement( sbXml, TAG_PORTLET_JSR170 );

            //        else
            //        {
            //            XmlUtil.beginElement( sbXml, TAG_WORKSPACES );
            //            currentView = _nViewId;
            //
            //            ReferenceList listWorkspaces = instance.getAvailableWorkspaces(  );
            //
            //            for ( ReferenceItem item : listWorkspaces )
            //            {
            //                if ( instance.canAccess( item.getCode(  ), IWorkspace.READ_ACCESS, getUserRoles( request ) ) )
            //                {
            //                    // this is the default workspace
            //                    if ( _nViewId == -1 )
            //                    {
            //                        currentView = Integer.parseInt( item.getCode(  ) );
            //                    }
            //
            //                    XmlUtil.addElement( sbXml, TAG_WORKSPACE, item.getCode(  ) );
            //                }
            //                else
            //                {
            //                    if ( ( _nViewId != -1 ) && _strWorkspaceName.equals( item.getCode(  ) ) )
            //                    {
            //                        currentView = -1;
            //                    }
            //                }
            //            }
            //
            //            XmlUtil.endElement( sbXml, TAG_WORKSPACES );
            //        }
            XmlUtil.addElement( sbXml, TAG_CURRENT_WORKSPACE, _view.getName(  ) );
            XmlUtil.addElement( sbXml, TAG_DIRECTORY_PATH, currentDir.getAbsolutePath(  ) );
            sbXml.append( getBreadCrumbs( currentWorkspace, currentDir.getAbsolutePath(  ),
                    adminWorkspace.getJcrType(  ), _view ) );

            boolean canRead = false;
            boolean canWrite = false;
            boolean canRemove = false;

            if ( currentWorkspace != null )
            {
                // check some rights
                Map availableAccess = instance.getAvailableAccess( currentWorkspace, getUserRoles( request ) );
                canRead = (Boolean) availableAccess.get( IWorkspace.READ_ACCESS );
                canWrite = (Boolean) availableAccess.get( IWorkspace.WRITE_ACCESS );
                canRemove = (Boolean) availableAccess.get( IWorkspace.REMOVE_ACCESS );

                if ( canRead )
                {
                    for ( IRepositoryFile repoFile : (List<IRepositoryFile>) _jcrRestrictedOperation.doRestrictedOperation( 
                            RestrictedRepositoryFileHome.getRepositoryFileList( instance, currentWorkspace,
                                currentDir.getAbsolutePath(  ) ) ) )
                    {
                        sbXml.append( repoFile.getXml(  ) );
                    }
                }
            }

            XmlUtil.addElement( sbXml, TAG_CAN_READ, String.valueOf( canRead ) );
            XmlUtil.addElement( sbXml, TAG_CAN_WRITE, String.valueOf( canWrite ) );
            XmlUtil.addElement( sbXml, TAG_CAN_REMOVE, String.valueOf( canRemove ) );
            XmlUtil.endElement( sbXml, TAG_PORTLET_JSR170 );
        }
        catch ( Exception e )
        {
            AppLogService.debug( e );

            StringBuffer sbErrorXml = new StringBuffer(  );
            XmlUtil.beginElement( sbErrorXml, TAG_PORTLET_JSR170_ERROR );
            XmlUtil.addElement( sbErrorXml, TAG_ERROR, e.getLocalizedMessage(  ) );
            XmlUtil.endElement( sbErrorXml, TAG_PORTLET_JSR170_ERROR );

            return addPortletTags( sbErrorXml );
        }

        return addPortletTags( sbXml );
    }

    /**
     * Returns the Xml code of the FolderListing portlet with XML heading
     *
     * @param request the HttpServletRequest
     * @return the Xml code of the FolderListing portlet
     */
    public String getXmlDocument( HttpServletRequest request )
    {
        return XmlUtil.getXmlHeader(  ) + getXml( request );
    }

    /**
     * Update the portlet
     */
    public void update(  )
    {
        Jsr170PortletHome.getInstance(  ).update( this );
    }

    /**
     * Remove portlet
     */
    public void remove(  )
    {
        Jsr170PortletHome.getInstance(  ).remove( this );
    }

    /**
     * Get portlet and workspace parameters from request
     * @param request the request
     */
    private void getParameters( HttpServletRequest request )
    {
        String strIdPortlet = request.getParameter( Parameters.PORTLET_ID );
        //        _nCurrentView = request.getParameter( PARAMETER_VIEW + "_" + strIdPortlet );
        _strFileId = request.getParameter( PARAMETER_FILE_ID + "_" + this.getId(  ) );
        _plugin = PluginService.getPlugin( "jsr170" );
        _view = AdminJcrHome.getInstance(  ).findViewById( _nDefaultView, _plugin );
        _jcrRestrictedOperation = new JcrRestrictedOperation( _view.getUser(  ), _view.getPassword(  ) );
    }

    /**
     * Get user roles from request
     * @param request the request
     * @return an array containing the user roles
     */
    private String[] getUserRoles( HttpServletRequest request )
    {
        LuteceUser user = SecurityService.getInstance(  ).getRegisteredUser( request );

        return ( user == null ) ? new String[] { "none" } : user.getRoles(  );
    }

    public void setDefaultView( int nViewId )
    {
        _hasDefaultView = nViewId > -1;
        _nDefaultView = nViewId;
    }

    public int getDefaultView(  )
    {
        return _nDefaultView;
    }

    public boolean hasDefaultView(  )
    {
        return _hasDefaultView;
    }

    private StringBuffer getBreadCrumbs( final String strWorkspace, final String strPath, String strJcrType,
        AdminView view )
    {
        final RepositoryFileHome instance = RepositoryFileHome.getInstance( strJcrType );

        StringBuffer sb = new StringBuffer(  );
        XmlUtil.beginElement( sb, TAG_BREADCRUMBS );

        String strAbsolutePath = "/";

        for ( String pathElement : strPath.split( "/" ) )
        {
            if ( pathElement.equals( "" ) )
            {
                continue;
            }

            strAbsolutePath += ( "/" + pathElement );

            IRepositoryFile folder = (IRepositoryFile) _jcrRestrictedOperation.doRestrictedOperation( RestrictedRepositoryFileHome.getRepositoryFile( 
                        instance, strWorkspace, strAbsolutePath ) );

            if ( folder.isDirectory(  ) )
            {
                XmlUtil.beginElement( sb, TAG_BREADCRUMB );
                XmlUtil.addElement( sb, TAG_BREADCRUMB_NAME, folder.getName(  ) );
                XmlUtil.addElement( sb, TAG_BREADCRUMB_PATH, folder.getResourceId(  ) );
                XmlUtil.endElement( sb, TAG_BREADCRUMB );
            }
        }

        XmlUtil.endElement( sb, TAG_BREADCRUMBS );

        return sb;
    }
}
