/*
 * Copyright (c) 2002-2007, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.jcr.web;

import fr.paris.lutece.plugins.jcr.business.IWorkspace;
import fr.paris.lutece.plugins.jcr.business.RepositoryFileHome;
import fr.paris.lutece.plugins.jcr.business.admin.AdminJcrHome;
import fr.paris.lutece.plugins.jcr.business.admin.AdminView;
import fr.paris.lutece.plugins.jcr.business.admin.AdminWorkspace;
import fr.paris.lutece.plugins.jcr.business.portlet.Jsr170PortletHome;
import fr.paris.lutece.portal.business.role.RoleHome;
import fr.paris.lutece.portal.service.html.EncodingService;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.service.workgroup.AdminWorkgroupService;
import fr.paris.lutece.portal.web.admin.PluginAdminPageJspBean;
import fr.paris.lutece.portal.web.constants.Messages;
import fr.paris.lutece.portal.web.constants.Parameters;
import fr.paris.lutece.util.ReferenceItem;
import fr.paris.lutece.util.ReferenceList;
import fr.paris.lutece.util.html.HtmlTemplate;
import fr.paris.lutece.util.url.UrlItem;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


/**
 * This class provides methods needed to serve the user a jsr170 file
 */
public class RepositoryFileJspBean extends PluginAdminPageJspBean
{
    //////////////////////////////////////////////////////////////////////////
    // Constants
    public static final int FILE_NOT_FOUND = 0;
    public static final int FILE_NOT_ALLOWED = 1;
    public static final String RIGHT_JSR170_MANAGEMENT = "JSR170_MANAGEMENT";

    // Parameters
    private static final String PARAMETER_FILE_NAME = "file";
    private static final String PARAMETER_DIRECTORY_NAME = "folder";
    private static final String PARAMETER_PORTLET_ID = "portlet_id";
    private static final String PARAMETER_PAGE_ID = "page_id";
    private static final String PARAMETER_MESSAGE_ERROR = "error";

    //JSP
    private static final String PROPERTY_ERROR_NOT_FOUND = "jsr170.error.NotFound";
    private static final String PROPERTY_ERROR_NOT_ALLOWED = "jsr170.error.NotAllowed";
    private static final String MARK_WORKSPACES_LIST = "workspaces_list";
    private static final String TEMPLATE_MANAGE_WORKSPACES = "/admin/plugins/jsr170/manage_workspaces.html";
    private static final String TEMPLATE_ADD_WORKSPACE = "/admin/plugins/jsr170/add_workspace.html";
    private static final String JSP_MANAGE_WORKSPACES = "ManageWorkspaces.jsp";
    private static final String PARAMETER_WORKSPACE_NAME = "workspace_name";
    private static final String TEMPLATE_MODIFY_WORKSPACE = "/admin/plugins/jsr170/modify_workspace.html";
    private static final String MARK_WORKSPACE_NAME = "workspace_name";
    private static final String MARK_AVAILABLE_ROLES = "available_roles";
    private static final String PARAMETER_READ_ROLES = "read_roles";
    private static final String MARK_WORKSPACE_READ_ROLES = "workspace_read_roles";
    private static final String MARK_WORKSPACE_WRITE_ROLES = "workspace_write_roles";
    private static final String MARK_WORKSPACE_REMOVE_ROLES = "workspace_remove_roles";
    private static final String PARAMETER_WRITE_ROLES = "write_roles";
    private static final String PARAMETER_REMOVE_ROLES = "remove_roles";
    private static final String PROPERTY_MANAGE_WORKSPACE = "jsr170.workspaces.pageTitle";
    private static final String PROPERTY_MANAGE_VIEW = "jsr170.views.pageTitle";
    private static final String MARK_VIEW_LIST = "views_list";
    private static final String TEMPLATE_MANAGE_VIEWS = "/admin/plugins/jsr170/manage_views.html";
    private static final String PARAMETER_JCR_TYPE = "jcr_type";
    private static final String PARAMETER_WORKGROUP = "workgroup";
    private static final String PARAMETER_WORKSPACE_ID = "workspace_id";
    private static final String PARAMETER_VIEW_NAME = "view_name";
    private static final String PARAMETER_USER = "view_user";
    private static final String PARAMETER_PASSWORD = "view_password";
    private static final String JSP_MANAGE_VIEWS = "ManageViews.jsp";
    private static final String TEMPLATE_ADD_VIEW = "/admin/plugins/jsr170/add_view.html";
    private static final String MARK_JCR_LIST = "jcr_list";
    private static final String MARK_WORKGROUP_LIST = "workgroup_list";
    private static final String MARK_WORKSPACE_ID = "workspace_id";
    private static final String PARAMETER_WORKSPACE_LABEL = "workspace_label";
    private static final String PARAMETER_VIEW_ID = "view_id";
    private static final String MARK_VIEW = "view";
    private static final String TEMPLATE_MODIFY_VIEW = "/admin/plugins/jsr170/modify_view.html";
    private static final String TEMPLATE_MODIFY_WORKSPACE_ROLES = "/admin/plugins/jsr170/modify_workspace_roles.html";
    private static final String MARK_WORKSPACE = "workspace";
    private static final String MESSAGE_EXISTING_PORTLETS = "jsr170.message.existingPortlets";
    private static final String MESSAGE_EXISTING_VIEWS = "jsr170.message.existingViews";
    private static final String PATH_JSP = "jsp/admin/plugins/jsr170/";
    private static final String JSP_DELETE_WORKSPACE = "DoDeleteWorkspace.jsp";
    private static final String MESSAGE_CONFIRM_DELETE_WORKSPACE = "jsr170.message.confirmDeleteWorkspace";
    private static final String JSP_DELETE_VIEW = "DoDeleteView.jsp";
    private static final String MESSAGE_CONFIRM_DELETE_VIEW = "jsr170.message.confirmDeleteView";

    /**
     * Public constructor
     */
    public RepositoryFileJspBean(  )
    {
    }

    /**
     * Get the directory path of the file to display from the request
     *
     * @param request the http request
     *
     * @return the path of the directory containing the file to display (given from the root folder defined for the
     *         porltet)
     */
    public String getDirectoryPath( HttpServletRequest request )
    {
        String strDirPath = request.getParameter( PARAMETER_DIRECTORY_NAME );

        return strDirPath;
    }

    public String getFileErrorUrl( HttpServletRequest request, int nErrorType )
    {
        String strError = null;
        String strDirPath = request.getParameter( PARAMETER_DIRECTORY_NAME );
        String strPortletId = request.getParameter( PARAMETER_PORTLET_ID );
        String strPageId = request.getParameter( PARAMETER_PAGE_ID );

        switch ( nErrorType )
        {
            case FILE_NOT_FOUND:
                strError = I18nService.getLocalizedString( PROPERTY_ERROR_NOT_FOUND, request.getLocale(  ) );

                break;

            case FILE_NOT_ALLOWED:
                strError = I18nService.getLocalizedString( PROPERTY_ERROR_NOT_ALLOWED, request.getLocale(  ) );

                break;

            default:
                strError = I18nService.getLocalizedString( PROPERTY_ERROR_NOT_FOUND, request.getLocale(  ) );

                break;
        }

        strError = EncodingService.encodeUrl( strError );

        String strBaseUrl = AppPathService.getBaseUrl( request );
        String strParam = PARAMETER_MESSAGE_ERROR + "=" + strError + "&" + PARAMETER_PAGE_ID + "=" + strPageId + "&" +
            PARAMETER_DIRECTORY_NAME + "_" + strPortletId + "=" + strDirPath + "#" + PARAMETER_PORTLET_ID + "_" +
            strPortletId;
        String strUrl = strBaseUrl + AppPathService.getPortalUrl(  ) + "?" + strParam;

        return strUrl;
    }

    /**
     * Get the name of the file to display from the request
     *
     * @param request the http request
     *
     * @return the name of the file to display
     */
    public String getFilename( HttpServletRequest request )
    {
        String strFileName = request.getParameter( PARAMETER_FILE_NAME );

        return strFileName;
    }

    /**
     * Check that the user can view the file requested
     *
     * @param request the http request
     *
     * @return true if the user has the right to view the file, false otherwise
     */
    public boolean checkRights( HttpServletRequest request )
    {
        String strIdPortlet = request.getParameter( Parameters.PORTLET_ID );

        return true;
    }

    /**
     * Get the workspace management screen
     * @param request the request
     * @return html code
     */
    public String getManageWorkspaces( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_MANAGE_WORKSPACE );

        Collection<AdminWorkspace> availableWorkspaces = AdminJcrHome.getInstance(  ).findAllWorkspaces( getPlugin(  ) );
        Map<String, String> allJcrMap = new HashMap<String, String>(  );

        for ( ReferenceItem jcr : RepositoryFileHome.getAvailableJcr(  ) )
        {
            allJcrMap.put( jcr.getCode(  ), jcr.getName(  ) );
        }

        // find availables workspaces that work on available jcr
        Collection<AdminWorkspace> authorizedWorkspaces = new ArrayList<AdminWorkspace>(  );

        for ( AdminWorkspace workspace : (Collection<AdminWorkspace>) AdminWorkgroupService.getAuthorizedCollection( 
                availableWorkspaces, getUser(  ) ) )
        {
            if ( allJcrMap.containsKey( workspace.getJcrType(  ) ) )
            {
                authorizedWorkspaces.add( workspace );
            }
        }

        // transform referenceList of jcr to a map
        Map<String, String> jcrMap = new HashMap<String, String>(  );

        for ( ReferenceItem jcr : getModifiableJcr(  ) )
        {
            jcrMap.put( jcr.getCode(  ), jcr.getName(  ) );
        }

        HashMap model = new HashMap(  );
        model.put( MARK_WORKSPACES_LIST, authorizedWorkspaces );
        model.put( MARK_JCR_LIST, jcrMap );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_MANAGE_WORKSPACES, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Get the workspace management screen
     * @param request the request
     * @return html code
     */
    public String getManageViews( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_MANAGE_VIEW );

        Map<String, AdminWorkspace> availableWorkspaces = AdminJcrHome.getInstance(  ).getWorkspacesList( getPlugin(  ) );
        Collection<AdminView> adminViewsList = AdminJcrHome.getInstance(  ).findAllViews( getPlugin(  ) );

        HashMap model = new HashMap(  );
        model.put( MARK_WORKSPACES_LIST, availableWorkspaces );
        model.put( MARK_VIEW_LIST, AdminWorkgroupService.getAuthorizedCollection( adminViewsList, getUser(  ) ) );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_MANAGE_VIEWS, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Get the workspace creation screen
     * @param request the request
     * @return html code
     */
    public String getAddWorkspace( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_MANAGE_WORKSPACE );

        HashMap model = new HashMap(  );
        model.put( MARK_JCR_LIST, getModifiableJcr(  ) );
        model.put( MARK_WORKGROUP_LIST, AdminWorkgroupService.getUserWorkgroups( getUser(  ), getLocale(  ) ) );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_ADD_WORKSPACE, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Create a new workspace
     * @param request the request
     * @return
     */
    public String doAddWorkspace( HttpServletRequest request )
    {
        String strWorkspaceName = request.getParameter( PARAMETER_WORKSPACE_NAME );
        String strWorkspaceLabel = request.getParameter( PARAMETER_WORKSPACE_LABEL );
        String strJcrType = request.getParameter( PARAMETER_JCR_TYPE );
        String strWorkgroup = request.getParameter( PARAMETER_WORKGROUP );

        String strValidation = validateWorkspaceInput( strWorkspaceName, strWorkspaceLabel, strJcrType, strWorkgroup,
                request );

        if ( strValidation != null )
        {
            return strValidation;
        }

        RepositoryFileHome repositoryFileHome = RepositoryFileHome.getInstance( strJcrType );

        if ( repositoryFileHome.canCreateWorkspace(  ) )
        {
            repositoryFileHome.createWorkspace( strWorkspaceName );

            AdminWorkspace workspace = new AdminWorkspace(  );
            workspace.setJcrType( strJcrType );
            workspace.setName( strWorkspaceName );
            workspace.setWorkgroup( strWorkgroup );
            workspace.setLabel( strWorkspaceLabel );
            AdminJcrHome.getInstance(  ).createWorkspace( workspace, getPlugin(  ) );
        }
        else
        {
            //TODO do something else
        }

        return JSP_MANAGE_WORKSPACES;
    }

    /**
     * Get the workspace creation screen
     * @param request the request
     * @return html code
     */
    public String getAddView( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_MANAGE_VIEW );

        Collection<AdminWorkspace> availableWorkspaces = AdminJcrHome.getInstance(  ).findAllWorkspaces( getPlugin(  ) );
        HashMap model = new HashMap(  );
        model.put( MARK_WORKSPACES_LIST,
            AdminWorkgroupService.getAuthorizedCollection( availableWorkspaces, getUser(  ) ) );
        model.put( MARK_WORKGROUP_LIST, AdminWorkgroupService.getUserWorkgroups( getUser(  ), getLocale(  ) ) );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_ADD_VIEW, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Create a new workspace
     * @param request the request
     * @return
     */
    public String doAddView( HttpServletRequest request )
    {
        String strWorkspaceId = request.getParameter( PARAMETER_WORKSPACE_ID );
        String strViewName = request.getParameter( PARAMETER_VIEW_NAME );
        String strWorkgroup = request.getParameter( PARAMETER_WORKGROUP );
        String strUser = request.getParameter( PARAMETER_USER );
        String strPassword = request.getParameter( PARAMETER_PASSWORD );

        String validationResult = validateViewInput( strWorkspaceId, strViewName, strWorkgroup, strUser, strPassword,
                request );

        if ( validationResult != null )
        {
            return validationResult;
        }

        AdminView adminView = new AdminView(  );
        adminView.setName( strViewName );
        adminView.setUser( strUser );
        adminView.setPassword( strPassword );
        adminView.setWorkgroup( strWorkgroup );
        adminView.setWorkspaceId( Integer.parseInt( strWorkspaceId ) );

        AdminJcrHome.getInstance(  ).createView( adminView, getPlugin(  ) );

        return JSP_MANAGE_VIEWS;
    }

    /**
     * Get the workspace modification screen
     * @param request the request
     * @return html code
     */
    public String getModifyWorkspaceRoles( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_MANAGE_WORKSPACE );

        final String strWorkspaceId = request.getParameter( PARAMETER_WORKSPACE_ID );
        AdminWorkspace adminWorkspace = AdminJcrHome.getInstance(  )
                                                    .findWorkspaceById( Integer.parseInt( strWorkspaceId ),
                getPlugin(  ) );
        IWorkspace workspace = RepositoryFileHome.getInstance( adminWorkspace.getJcrType(  ) )
                                                 .getWorkspace( adminWorkspace.getName(  ) );

        ReferenceList allRoles = RoleHome.getRolesList(  );
        ReferenceList readRoles = RoleHome.getRolesList(  );
        ReferenceList writeRoles = RoleHome.getRolesList(  );
        ReferenceList deleteRoles = RoleHome.getRolesList(  );
        readRoles.checkItems( workspace.getRoles( IWorkspace.READ_ACCESS ) );
        writeRoles.checkItems( workspace.getRoles( IWorkspace.WRITE_ACCESS ) );
        deleteRoles.checkItems( workspace.getRoles( IWorkspace.REMOVE_ACCESS ) );

        HashMap model = new HashMap(  );
        model.put( MARK_WORKSPACE_ID, adminWorkspace.getId(  ) );
        model.put( MARK_WORKSPACE_NAME, workspace.getName(  ) );
        model.put( MARK_WORKSPACE_READ_ROLES, readRoles );
        model.put( MARK_WORKSPACE_WRITE_ROLES, writeRoles );
        model.put( MARK_WORKSPACE_REMOVE_ROLES, deleteRoles );
        model.put( MARK_AVAILABLE_ROLES, allRoles );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_MODIFY_WORKSPACE_ROLES, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    public String doModifyWorkspaceRoles( HttpServletRequest request )
    {
        final String strWorkspaceId = request.getParameter( PARAMETER_WORKSPACE_ID );
        final String[] readRoles = request.getParameterValues( PARAMETER_READ_ROLES );
        final String[] writeRoles = request.getParameterValues( PARAMETER_WRITE_ROLES );
        final String[] removeRoles = request.getParameterValues( PARAMETER_REMOVE_ROLES );

        AdminWorkspace adminWorkspace = AdminJcrHome.getInstance(  )
                                                    .findWorkspaceById( Integer.parseInt( strWorkspaceId ),
                getPlugin(  ) );
        RepositoryFileHome instance = RepositoryFileHome.getInstance( adminWorkspace.getJcrType(  ) );
        IWorkspace workspace = instance.getWorkspace( adminWorkspace.getName(  ) );
        workspace.setRoles( IWorkspace.READ_ACCESS, readRoles );
        workspace.setRoles( IWorkspace.WRITE_ACCESS, writeRoles );
        workspace.setRoles( IWorkspace.REMOVE_ACCESS, removeRoles );
        instance.modify( workspace );

        return JSP_MANAGE_WORKSPACES;
    }

    /**
     * Get the workspace creation screen
     * @param request the request
     * @return html code
     */
    public String getModifyView( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_MANAGE_VIEW );

        String strViewId = request.getParameter( PARAMETER_VIEW_ID );

        Collection<AdminWorkspace> availableWorkspaces = AdminJcrHome.getInstance(  ).findAllWorkspaces( getPlugin(  ) );
        ReferenceList availableList = ReferenceList.convert( AdminWorkgroupService.getAuthorizedCollection( 
                    availableWorkspaces, getUser(  ) ), "id", "label", true );
        HashMap model = new HashMap(  );
        model.put( MARK_WORKSPACES_LIST, availableList );
        model.put( MARK_VIEW, AdminJcrHome.getInstance(  ).findViewById( Integer.parseInt( strViewId ), getPlugin(  ) ) );
        model.put( MARK_WORKGROUP_LIST, AdminWorkgroupService.getUserWorkgroups( getUser(  ), getLocale(  ) ) );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_MODIFY_VIEW, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Create a new workspace
     * @param request the request
     * @return
     */
    public String doModifyView( HttpServletRequest request )
    {
        String strViewId = request.getParameter( PARAMETER_VIEW_ID );
        String strWorkspaceId = request.getParameter( PARAMETER_WORKSPACE_ID );
        String strViewName = request.getParameter( PARAMETER_VIEW_NAME );
        String strWorkgroup = request.getParameter( PARAMETER_WORKGROUP );
        String strUser = request.getParameter( PARAMETER_USER );
        String strPassword = request.getParameter( PARAMETER_PASSWORD );

        String validationResult = validateViewInput( strWorkspaceId, strViewName, strWorkgroup, strUser, strPassword,
                request );

        if ( validationResult != null )
        {
            return validationResult;
        }

        AdminView adminView = new AdminView(  );
        adminView.setId( Integer.parseInt( strViewId ) );
        adminView.setName( strViewName );
        adminView.setUser( strUser );
        adminView.setPassword( strPassword );
        adminView.setWorkgroup( strWorkgroup );
        adminView.setWorkspaceId( Integer.parseInt( strWorkspaceId ) );

        AdminJcrHome.getInstance(  ).modifyView( adminView, getPlugin(  ) );

        return JSP_MANAGE_VIEWS;
    }

    /**
     * Get the workspace creation screen
     * @param request the request
     * @return html code
     */
    public String getModifyWorkspace( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_MANAGE_WORKSPACE );

        String strWorkspaceId = request.getParameter( PARAMETER_WORKSPACE_ID );

        HashMap model = new HashMap(  );
        model.put( MARK_JCR_LIST, getModifiableJcr(  ) );
        model.put( MARK_WORKGROUP_LIST, AdminWorkgroupService.getUserWorkgroups( getUser(  ), getLocale(  ) ) );
        model.put( MARK_WORKSPACE,
            AdminJcrHome.getInstance(  ).findWorkspaceById( Integer.parseInt( strWorkspaceId ), getPlugin(  ) ) );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_MODIFY_WORKSPACE, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Create a new workspace
     * @param request the request
     * @return
     */
    public String doModifyWorkspace( HttpServletRequest request )
    {
        String strWorkspaceId = request.getParameter( PARAMETER_WORKSPACE_ID );
        String strWorkspaceName = request.getParameter( PARAMETER_WORKSPACE_NAME );
        String strWorkspaceLabel = request.getParameter( PARAMETER_WORKSPACE_LABEL );
        String strJcrType = request.getParameter( PARAMETER_JCR_TYPE );
        String strWorkgroup = request.getParameter( PARAMETER_WORKGROUP );

        String strValidation = validateWorkspaceInput( strWorkspaceName, strWorkspaceLabel, strJcrType, strWorkgroup,
                request );

        if ( strValidation != null )
        {
            return strValidation;
        }

        AdminWorkspace workspace = new AdminWorkspace(  );
        workspace.setId( Integer.parseInt( strWorkspaceId ) );
        workspace.setJcrType( strJcrType );
        workspace.setName( strWorkspaceName );
        workspace.setWorkgroup( strWorkgroup );
        workspace.setLabel( strWorkspaceLabel );
        AdminJcrHome.getInstance(  ).modifyWorkspace( workspace, getPlugin(  ) );

        return JSP_MANAGE_WORKSPACES;
    }

    /**
     * View deletion confirmation
     * @param request the request
     * @return an url
     */
    public String deleteView( HttpServletRequest request )
    {
        String strViewId = request.getParameter( PARAMETER_VIEW_ID );
        AdminView view = AdminJcrHome.getInstance(  ).findViewById( Integer.parseInt( strViewId ), getPlugin(  ) );
        String[] messageArgs = { view.getName(  ) };
        UrlItem url = new UrlItem( PATH_JSP + JSP_DELETE_VIEW );
        url.addParameter( PARAMETER_VIEW_ID, strViewId );

        return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRM_DELETE_VIEW, messageArgs, url.getUrl(  ),
            AdminMessage.TYPE_CONFIRMATION );
    }

    /**
     * Deletes a view after having done some verification
     * @param request the request
     * @return the url
     */
    public String doDeleteView( HttpServletRequest request )
    {
        String strViewId = request.getParameter( PARAMETER_VIEW_ID );
        int nViewId = Integer.parseInt( strViewId );

        AdminView view = AdminJcrHome.getInstance(  ).findViewById( nViewId, getPlugin(  ) );

        if ( !AdminWorkgroupService.isAuthorized( view, getUser(  ) ) )
        {
            return AdminMessageService.getMessageUrl( request, Messages.USER_ACCESS_DENIED, AdminMessage.TYPE_STOP );
        }

        if ( ( (Jsr170PortletHome) Jsr170PortletHome.getInstance(  ) ).existsPortletWithViewId( nViewId ) )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_EXISTING_PORTLETS, AdminMessage.TYPE_STOP );
        }

        AdminJcrHome.getInstance(  ).deleteView( nViewId, getPlugin(  ) );

        return JSP_MANAGE_VIEWS;
    }

    /**
     * Workspace deletion confirmation
     * @param request the request
     * @return an url
     */
    public String deleteWorkspace( HttpServletRequest request )
    {
        String strWorkspaceId = request.getParameter( PARAMETER_WORKSPACE_ID );
        AdminWorkspace workspace = AdminJcrHome.getInstance(  )
                                               .findWorkspaceById( Integer.parseInt( strWorkspaceId ), getPlugin(  ) );
        String[] messageArgs = { workspace.getLabel(  ) };
        UrlItem url = new UrlItem( PATH_JSP + JSP_DELETE_WORKSPACE );
        url.addParameter( PARAMETER_WORKSPACE_ID, strWorkspaceId );

        return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRM_DELETE_WORKSPACE, messageArgs,
            url.getUrl(  ), AdminMessage.TYPE_CONFIRMATION );
    }

    /**
     * @param request the request
     * @return the url
     */
    public String doDeleteWorkspace( HttpServletRequest request )
    {
        String strWorkspaceId = request.getParameter( PARAMETER_WORKSPACE_ID );
        int nWorkspaceId = Integer.parseInt( strWorkspaceId );
        AdminJcrHome adminJcrHome = AdminJcrHome.getInstance(  );

        AdminWorkspace workspace = adminJcrHome.findWorkspaceById( nWorkspaceId, getPlugin(  ) );
        RepositoryFileHome repositoryFileHome = RepositoryFileHome.getInstance( workspace.getJcrType(  ) );

        // check if user is allowed and if workspaces are deletable
        if ( !AdminWorkgroupService.isAuthorized( workspace, getUser(  ) ) ||
                !repositoryFileHome.canCreateWorkspace(  ) )
        {
            return AdminMessageService.getMessageUrl( request, Messages.USER_ACCESS_DENIED, AdminMessage.TYPE_STOP );
        }

        if ( adminJcrHome.existsViewWithWorkspaceId( nWorkspaceId, getPlugin(  ) ) )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_EXISTING_VIEWS, AdminMessage.TYPE_STOP );
        }

        RepositoryFileHome.getInstance( workspace.getJcrType(  ) ).removeWorkspace( workspace.getName(  ) );
        AdminJcrHome.getInstance(  ).deleteWorkspace( nWorkspaceId, getPlugin(  ) );

        return JSP_MANAGE_WORKSPACES;
    }

    /**
     * Control view input screen
     * @param strWorkspaceId workspace id
     * @param strViewName view name
     * @param strWorkgroup workgroup
     * @param strUser user
     * @param strPassword password
     * @param request the request
     * @return
     */
    private String validateViewInput( String strWorkspaceId, String strViewName, String strWorkgroup, String strUser,
        String strPassword, HttpServletRequest request )
    {
        if ( ( strWorkspaceId == null ) || "".equals( strWorkspaceId ) )
        {
            return AdminMessageService.getMessageUrl( request, Messages.MANDATORY_FIELDS, AdminMessage.TYPE_STOP );
        }

        if ( ( strViewName == null ) || "".equals( strViewName ) )
        {
            return AdminMessageService.getMessageUrl( request, Messages.MANDATORY_FIELDS, AdminMessage.TYPE_STOP );
        }

        if ( ( strWorkgroup == null ) || "".equals( strWorkgroup ) )
        {
            return AdminMessageService.getMessageUrl( request, Messages.MANDATORY_FIELDS, AdminMessage.TYPE_STOP );
        }

        if ( ( strUser == null ) || "".equals( strUser ) )
        {
            return AdminMessageService.getMessageUrl( request, Messages.MANDATORY_FIELDS, AdminMessage.TYPE_STOP );
        }

        if ( ( strPassword == null ) || "".equals( strPassword ) )
        {
            return AdminMessageService.getMessageUrl( request, Messages.MANDATORY_FIELDS, AdminMessage.TYPE_STOP );
        }

        return null;
    }

    /**
     * @param strWorkspaceName the workspace name
     * @param strWorkspaceLabel the workspace description
     * @param strJcrType the jcr type
     * @param strWorkgroup the allowed workgroup
     * @param request the request
     * @return
     */
    private String validateWorkspaceInput( String strWorkspaceName, String strWorkspaceLabel, String strJcrType,
        String strWorkgroup, HttpServletRequest request )
    {
        if ( ( strWorkspaceName == null ) || "".equals( strWorkspaceName ) )
        {
            return AdminMessageService.getMessageUrl( request, Messages.MANDATORY_FIELDS, AdminMessage.TYPE_STOP );
        }

        if ( ( strWorkspaceLabel == null ) || "".equals( strWorkspaceLabel ) )
        {
            return AdminMessageService.getMessageUrl( request, Messages.MANDATORY_FIELDS, AdminMessage.TYPE_STOP );
        }

        if ( ( strWorkgroup == null ) || "".equals( strWorkgroup ) )
        {
            return AdminMessageService.getMessageUrl( request, Messages.MANDATORY_FIELDS, AdminMessage.TYPE_STOP );
        }

        if ( ( strJcrType == null ) || "".equals( strJcrType ) )
        {
            return AdminMessageService.getMessageUrl( request, Messages.MANDATORY_FIELDS, AdminMessage.TYPE_STOP );
        }

        return null;
    }

    /**
     *
     * @return a referenceList of modifiable workspaces
     */
    private ReferenceList getModifiableJcr(  )
    {
        ReferenceList jcrList = new ReferenceList(  );

        for ( ReferenceItem jcr : RepositoryFileHome.getAvailableJcr(  ) )
        {
            if ( RepositoryFileHome.getInstance( jcr.getCode(  ) ).canCreateWorkspace(  ) )
            {
                jcrList.add( jcr );
            }
        }

        return jcrList;
    }
}
