/*
 * Copyright (c) 2002-2007, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.jcr.web.portlet;

import fr.paris.lutece.plugins.jcr.authentication.JcrRestrictedOperation;
import fr.paris.lutece.plugins.jcr.business.IRepositoryFile;
import fr.paris.lutece.plugins.jcr.business.RepositoryFileHome;
import fr.paris.lutece.plugins.jcr.business.RestrictedRepositoryFileHome;
import fr.paris.lutece.plugins.jcr.business.admin.AdminJcrHome;
import fr.paris.lutece.plugins.jcr.business.admin.AdminView;
import fr.paris.lutece.plugins.jcr.business.admin.AdminWorkspace;
import fr.paris.lutece.plugins.jcr.business.portlet.Jsr170Portlet;
import fr.paris.lutece.plugins.jcr.business.portlet.Jsr170PortletHome;
import fr.paris.lutece.portal.business.portlet.PortletHome;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.plugin.PluginService;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.service.workgroup.AdminWorkgroupService;
import fr.paris.lutece.portal.web.constants.Parameters;
import fr.paris.lutece.portal.web.portlet.PortletJspBean;
import fr.paris.lutece.portal.web.upload.MultipartHttpServletRequest;
import fr.paris.lutece.util.ReferenceList;
import fr.paris.lutece.util.html.HtmlTemplate;
import fr.paris.lutece.util.url.UrlItem;

import org.apache.commons.fileupload.FileItem;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import java.net.URLDecoder;

import java.util.Collection;
import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;


/**
 * This class provides the user interface to manage Jsr170 Portlet features
 */
public class Jsr170PortletJspBean extends PortletJspBean
{
    // Rights
    public static final String RIGHT_MANAGE_ADMIN_SITE = "CORE_ADMIN_SITE";
    public static final int FILE_NOT_FOUND = 0;
    public static final int FILE_NOT_ALLOWED = 1;

    // Messages
    private static final String MESSAGE_FOLDER_INVALID = "jsr170.message.folderInvalid";

    //Parameters
    private static final String PARAMETER_PAGE_ID = "page_id";
    private static final String PARAMETER_PORTLET_ID = "portlet_id";
    private static final String PARAMETER_PORTLET_TYPE_ID = "portlet_type_id";
    private static final String PARAMETER_WORKSPACE = "workspace";

    //Markers
    private static final String MARK_FOLDER_LIST = "folder_list";
    private static final String MARK_FOLDER_ID = "folder_id";

    // Templates
    private static final String TEMPLATE_COMBO_FOLDER = "admin/plugins/jsr170/portlet/combo_folder.html";
    private static final String COMBO_FOLDER_LIST = "@combo_folders@";
    private static final String PARAMETER_ALL_WORKSPACE = "all_authorized_workspaces";
    private static final String PARAMETER_VIEW = "view";
    private static final String PARAMETER_FILE_ID = "file_id";
    private String _strWorkspaceName = "default";
    private final Plugin _plugin = PluginService.getPlugin( "jsr170" );
    private Jsr170Portlet _portlet;
    private AdminView _view;
    private String _strIdPortlet;
    private String _strJcrType;
    private AdminWorkspace _workspace;
    private JcrRestrictedOperation _jcrRestrictedOperation;

    /**
     * Returns the jsr170 portlet creation form
     *
     * @param request The Http request
     *
     * @return The HTML form
     */
    public String getCreate( HttpServletRequest request )
    {
        String strIdPage = request.getParameter( PARAMETER_PAGE_ID );
        String strPortletIdType = request.getParameter( PARAMETER_PORTLET_TYPE_ID );

        Collection<AdminView> listViews = AdminJcrHome.getInstance(  ).findAllViews( _plugin );
        Collection filteredList = AdminWorkgroupService.getAuthorizedCollection( listViews, getUser(  ) );

        HashMap model = new HashMap(  );
        model.put( MARK_FOLDER_LIST, ReferenceList.convert( filteredList, "id", "name", true ) );
        model.put( MARK_FOLDER_ID, "" );

        HtmlTemplate template = getCreateTemplate( strIdPage, strPortletIdType, model );

        return template.getHtml(  );
    }

    /**
     * Returns the jsr170 portlet modification form
     *
     * @param request The Http request
     *
     * @return The HTML form
     */
    public String getModify( HttpServletRequest request )
    {
        String strPortletId = request.getParameter( PARAMETER_PORTLET_ID );
        int nPortletId = Integer.parseInt( strPortletId );
        Jsr170Portlet portlet = (Jsr170Portlet) PortletHome.findByPrimaryKey( nPortletId );
        Collection<AdminView> listViews = AdminJcrHome.getInstance(  ).findAllViews( _plugin );
        Collection filteredList = AdminWorkgroupService.getAuthorizedCollection( listViews, getUser(  ) );

        HashMap model = new HashMap(  );
        model.put( MARK_FOLDER_LIST, ReferenceList.convert( filteredList, "id", "name", true ) );
        model.put( MARK_FOLDER_ID, portlet.hasDefaultView(  ) ? portlet.getDefaultView(  ) : ( -1 ) );

        HtmlTemplate template = getModifyTemplate( portlet, model );

        return template.getHtml(  );
    }

    /**
     * Returns portlet's properties prefix
     *
     * @return prefix
     */
    public String getPropertiesPrefix(  )
    {
        return "portlet.jsr170";
    }

    /**
     * Process portlet's creation
     *
     * @param request The Http request
     *
     * @return The Jsp management URL of the process result
     */
    public String doCreate( HttpServletRequest request )
    {
        Jsr170Portlet portlet = new Jsr170Portlet(  );
        String strIdPage = request.getParameter( PARAMETER_PAGE_ID );
        String strViewId = request.getParameter( PARAMETER_VIEW );
        int nIdPage = Integer.parseInt( strIdPage );
        int nViewId = Integer.parseInt( strViewId );

        // get portlet common attributes
        String strErrorUrl = setPortletCommonData( request, portlet );

        if ( strErrorUrl != null )
        {
            return strErrorUrl;
        }

        portlet.setPageId( nIdPage );

        if ( strViewId != null )
        {
            portlet.setDefaultView( nViewId );
        }

        if ( isValidFolder( strViewId ) )
        {
            Jsr170PortletHome.getInstance(  ).create( portlet );
        }
        else
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_FOLDER_INVALID, AdminMessage.TYPE_STOP );
        }

        //Displays the page with the new Portlet
        return getPageUrl( nIdPage );
    }

    /**
     * Process portlet's modification
     *
     * @param request The Http request
     *
     * @return The Jsp management URL of the process result
     */
    public String doModify( HttpServletRequest request )
    {
        // Getting portlet
        String strPortletId = request.getParameter( PARAMETER_PORTLET_ID );
        int nPortletId = Integer.parseInt( strPortletId );
        Jsr170Portlet portlet = (Jsr170Portlet) PortletHome.findByPrimaryKey( nPortletId );
        String strViewId = request.getParameter( PARAMETER_VIEW );
        int nViewId = Integer.parseInt( strViewId );

        // get portlet common attributes
        String strErrorUrl = setPortletCommonData( request, portlet );

        if ( strErrorUrl != null )
        {
            return strErrorUrl;
        }

        // Getting portlet's common attributs
        setPortletCommonData( request, portlet );

        portlet.setDefaultView( nViewId );

        portlet.update(  );

        return getPageUrl( portlet.getPageId(  ) );
    }

    /**
     * Return the folder listing depending on rights
     *
     * @param listFolders The list of folders
     * @param strDefaultFolderId the default id of the folder
     *
     * @return DOCUMENT ME!
     */
    String getFolderIndexCombo( ReferenceList listFolders, String strDefaultFolderId )
    {
        HashMap model = new HashMap(  );
        model.put( MARK_FOLDER_LIST, listFolders );
        model.put( MARK_FOLDER_ID, strDefaultFolderId );

        HtmlTemplate templateCombo = AppTemplateService.getTemplate( TEMPLATE_COMBO_FOLDER, getLocale(  ), model );

        return templateCombo.getHtml(  );
    }

    /**
     * Verifies whether the folder is valid
     *
     * @param strFolderId The Id corresponding to the folder
     *
     * @return true if the folder is a valid folder
     */
    private boolean isValidFolder( String strFolderId )
    {
        //TODO: still necessary?
        return true;
    }

    /**
     * Get the binary content corresponding to the file to display
     *
     * @param request the http request
     *
     * @return the content to display
     */
    public byte[] getFileContent( HttpServletRequest request )
    {
        getParameters( request );

        String strFileId = request.getParameter( "file_id" );

        IRepositoryFile file = (IRepositoryFile) _jcrRestrictedOperation.doRestrictedOperation( RestrictedRepositoryFileHome.getRepositoryFileById( 
                    RepositoryFileHome.getInstance( _strJcrType ), _workspace.getName(  ), strFileId ) );

        if ( file.exists(  ) && file.isFile(  ) )
        {
            try
            {
                final InputStream in = file.getContent(  );

                final byte[] buf = new byte[1000];
                ByteArrayOutputStream out = new ByteArrayOutputStream(  );
                int len = 0;

                do
                {
                    len = in.read( buf );

                    if ( len == -1 )
                    {
                        break;
                    }

                    out.write( buf, 0, len );
                }
                while ( len > 0 );

                in.close(  );
                out.flush(  );

                return out.toByteArray(  );
            }
            catch ( FileNotFoundException e )
            {
                AppLogService.error( e.getMessage(  ), e );

                return null;
            }
            catch ( IOException e )
            {
                AppLogService.error( e.getMessage(  ), e );

                return null;
            }
        }

        return null;
    }

    /**
     * Upload a file in current workspace
     * @param request the request
     * @return the redirect url
     */
    public String uploadFile( HttpServletRequest request )
    {
        MultipartHttpServletRequest mRequest = (MultipartHttpServletRequest) request;
        FileItem item = mRequest.getFile( "upload_file" );
        getParameters( request );

        String strFolderName = request.getParameter( "directory_path" );

        if ( strFolderName == null )
        {
            strFolderName = "";
        }

        String strParentFileId = "";
        IRepositoryFile parentFile = null;

        try
        {
            File f = File.createTempFile( item.getName(  ), "" );
            item.write( f );

            RepositoryFileHome instance = RepositoryFileHome.getInstance( _strJcrType );
            _jcrRestrictedOperation.doRestrictedOperation( RestrictedRepositoryFileHome.addFile( instance,
                    _workspace.getName(  ), strFolderName + "/" + item.getName(  ), f ) );

            parentFile = (IRepositoryFile) _jcrRestrictedOperation.doRestrictedOperation( RestrictedRepositoryFileHome.getRepositoryFile( 
                        instance, _workspace.getName(  ), strFolderName ) );
        }
        catch ( Exception e )
        {
            throw new AppException( e.getLocalizedMessage(  ), e );
        }

        // reload portlet
        return getRedirectUrl( request, parentFile );
    }

    /**
     * Get filename from request
     * @param request the request
     * @return the filename
     */
    public String getFilename( HttpServletRequest request )
    {
        getParameters( request );

        String strFileId = request.getParameter( "file_id" );

        IRepositoryFile file = (IRepositoryFile) _jcrRestrictedOperation.doRestrictedOperation( RestrictedRepositoryFileHome.getRepositoryFileById( 
                    RepositoryFileHome.getInstance( _strJcrType ), _workspace.getName(  ), strFileId ) );

        return file.getName(  );
    }

    /**
     * Delete a file from current workspace
     * @param request the request
     * @return the redirect url
     */
    public String deleteFile( HttpServletRequest request )
    {
        // TODO : add confirmation screen
        getParameters( request );

        String strFileId = request.getParameter( PARAMETER_FILE_ID );
        RepositoryFileHome instance = RepositoryFileHome.getInstance( _strJcrType );
        IRepositoryFile currentFile = (IRepositoryFile) _jcrRestrictedOperation.doRestrictedOperation( RestrictedRepositoryFileHome.getRepositoryFileById( 
                    instance, _workspace.getName(  ), strFileId ) );

        _jcrRestrictedOperation.doRestrictedOperation( RestrictedRepositoryFileHome.removeRepositoryFile( instance,
                _workspace.getName(  ), currentFile.getAbsolutePath(  ) ) );

        String strPathFile = currentFile.getAbsolutePath(  );
        IRepositoryFile parentFile = (IRepositoryFile) _jcrRestrictedOperation.doRestrictedOperation( RestrictedRepositoryFileHome.getRepositoryFile( 
                    instance, _workspace.getName(  ), strPathFile.substring( 0, strPathFile.lastIndexOf( "/" ) ) ) );

        return getRedirectUrl( request, parentFile );
    }

    /**
     * Create a directory in current workspace
     * @param request the request
     * @return the redirect url
     */
    public String createDirectory( HttpServletRequest request )
    {
        getParameters( request );

        String strFolderName = request.getParameter( "directory_path" );
        String strNewDirectoryName = request.getParameter( "new_directory" );

        // TODO field control
        if ( strNewDirectoryName == null )
        {
            strNewDirectoryName = "";
        }

        String strAbsolutePath = strFolderName + "/" + strNewDirectoryName;
        IRepositoryFile parentFile = null;

        try
        {
            RepositoryFileHome instance = RepositoryFileHome.getInstance( _strJcrType );
            _jcrRestrictedOperation.doRestrictedOperation( RestrictedRepositoryFileHome.createFolder( instance,
                    _workspace.getName(  ), strAbsolutePath ) );
            parentFile = (IRepositoryFile) _jcrRestrictedOperation.doRestrictedOperation( RestrictedRepositoryFileHome.getRepositoryFile( 
                        instance, _workspace.getName(  ), strFolderName ) );
        }
        catch ( Exception e )
        {
            throw new AppException( e.getLocalizedMessage(  ), e );
        }

        // reload portlet
        return getRedirectUrl( request, parentFile );
    }

    /**
     * Select a new workspace
     * @param request the request
     * @return the url
     */
    public String changeWorkspace( HttpServletRequest request )
    {
        UrlItem url = AppPathService.resolveRedirectUrl( request, AppPathService.getPortalUrl(  ) );

        return url.getUrl(  );
    }

    /**
     * Get some parameter from request
     * @param request the request
     */
    private void getParameters( HttpServletRequest request )
    {
        _strIdPortlet = request.getParameter( Parameters.PORTLET_ID );
        _strWorkspaceName = request.getParameter( PARAMETER_WORKSPACE + "_" + _strIdPortlet );
        _portlet = (Jsr170Portlet) PortletHome.findByPrimaryKey( Integer.parseInt( _strIdPortlet ) );
        _view = AdminJcrHome.getInstance(  ).findViewById( _portlet.getDefaultView(  ), _plugin );
        _workspace = AdminJcrHome.getInstance(  ).findWorkspaceById( _view.getWorkspaceId(  ), _plugin );
        _strJcrType = AdminJcrHome.getInstance(  ).findJcrType( _portlet.getDefaultView(  ), _plugin );
        _jcrRestrictedOperation = new JcrRestrictedOperation( _view.getUser(  ), _view.getPassword(  ) );
    }

    private String getRedirectUrl( HttpServletRequest request, IRepositoryFile file )
    {
        UrlItem url = new UrlItem( AppPathService.getBaseUrl( request ) + AppPathService.getPortalUrl(  ) );
        String strResourceId = file.getResourceId(  );

        if ( !( "".equals( strResourceId ) || ( strResourceId == null ) ) )
        {
            url.addParameter( PARAMETER_FILE_ID + "_" + _strIdPortlet, strResourceId );
        }

        url.addParameter( Parameters.PORTLET_ID, _strIdPortlet );
        url.addParameter( PARAMETER_WORKSPACE + "_" + _strIdPortlet, _strWorkspaceName );
        url.addParameter( PARAMETER_PAGE_ID, request.getParameter( PARAMETER_PAGE_ID ) );

        return url.getUrl(  );
    }
}
